// DameLoesung.java                             MM 2009

/**
 * Eine Loesung eines Damenproblem
 * auf einem 8*8 Schachbrett von C. F. Gauss.
 * @author Monika Meiler
 */

public class DameLoesung
{
/* ------------------------------------------------- */
                                           // Attribute
/**
 * Loesungsfeld,
 * Spaltenkomponente enthaelt Zeilennummer,
 * in der eine Dame steht.
 */
  private int[] spalten = new int[ 8];

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Kontruktor, traegt Loesung ein.
 * @param sp Loesung als gewoehnliches Array
 */
  public DameLoesung( int[] sp)
  {
    for( int i = 0; i < 8; i++) spalten[ i] = sp[ i];
  }

/**
 * Kontruktor, fertigt eine Kopie einer Loesung an.
 * @param loesung DameLoesung
 */
  public DameLoesung( DameLoesung loesung)
  {
    for( int i = 0; i < 8; i++)
      spalten[ i] = loesung.spalten[ i];
  }

/* ------------------------------------------------- */
                                    // getData-Methoden
/**
 * Gibt Loesungsfeld zurck.
 */
  public int[] getSpalten()
  {
    return spalten;
  }

/* ------------------------------------------------- */
                           // Vergleichen von Loesungen
/**
 * Vergleich von Loesungen,
 * unabhaengig von Rotationen und Spiegelung.
 * @param obj Objekt, mit dem verglichen wird
 * @return true bei Gleichheit, sonst false
 */
  public boolean equals( Object obj)
  {
    if( obj == null) return false;
    if( obj == this) return true;
    if( getClass() != obj.getClass()) return false;

    DameLoesung loesung =                      // Kopie
      new DameLoesung((DameLoesung)obj);

    if( istGleich( loesung)) return true;
    if( istRotationsGleich( loesung)) return true;
    if( istSpiegelGleich( loesung)) return true;
    if( istRotationsGleich( loesung)) return true;

    return false;
  }

/**
 * Vergleich von Loesungen.
 * @param loesung Objekt, mit dem verglichen wird
 * @return true bei Gleichheit, sonst false
 */
  private boolean istGleich( DameLoesung loesung)
  {
    for( int i = 0; i < 8; i++)
      if( spalten[ i] != loesung.spalten[ i])
        return false;
    return true;
  }

/**
 * Rotation einer Loesung.
 */
  private void rotieren()
  {
    // Rotieren
    int spaltenNeu[ ] = new int[ 8];
    for( int i = 0; i < 8; i++)
      spaltenNeu[ 7 - spalten[ i]] = i;

    // Aktualisieren
    spalten = spaltenNeu;
  }

/**
 * Rotationsgleichheit.
 * @param loesung Objekt, mit dem verglichen wird
 * @return true bei Gleichheit, sonst false
 */
  private boolean istRotationsGleich
  ( DameLoesung loesung)
  {
    for( int i = 0; i < 3; i++)
    {
      loesung.rotieren();
      if( istGleich( loesung)) return true;
    }
    return false;
  }

/**
 * Spiegelung an der x-Achse.
 */
  private void spiegeln()
  {
    for( int i = 0; i < 8; i++)
      spalten[ i] = 7 - spalten[ i];
  }

/**
 * Spiegelgleichheit.
 * @param loesung Objekt, mit dem verglichen wird
 * @return true bei Gleichheit, sonst false
 */
  private boolean istSpiegelGleich
  ( DameLoesung loesung)
  {
    loesung.spiegeln();
    if( istGleich( loesung)) return true;
    return false;
  }

/**
 * Gleiche Objekte muessen den gleichen HashCode haben,
 * bei verschiedenen ist das ohne Bedeutung.
 * @return Hashcode 1
 */
  public int hashCode()
  {
    return 1;
  }

/* ------------------------------------------------- */
/**
 * Schachuebliche Darstellung einer Loesung.
 * @return Loesung
 */
  public String toString()
  {
    String str = "";
    for( int i = 0; i < 8; i++)
      str += " " + (char)('A' + i) + ( 1 + spalten[ i]);
    return str;
  }

/* ------------------------------------------------- */
/**
 * Test der Klasse DameLoesung.
 */
  public static void main( String[] args)
  {
    int[] loes1 = { 7, 3, 0, 2, 5, 1, 6, 4};// Original
    int[] loes2 = { 0, 6, 4, 7, 1, 3, 5, 2};// Rotation
    int[] loes3 = { 0, 4, 7, 5, 2, 6, 1, 3};// Spiegel
    int[] loes4 = { 7, 1, 3, 0, 6, 4, 2, 5};// Rot+Sp
    int[] loes5 = { 2, 5, 3, 1, 7, 4, 6, 0};// DiagSp
    int[] loes6 = { 1, 5, 3, 2, 7, 4, 6, 0};// ungleich

    DameLoesung loesung1 = new DameLoesung( loes1);
    DameLoesung loesung2 = new DameLoesung( loes2);
    DameLoesung loesung3 = new DameLoesung( loes3);
    DameLoesung loesung4 = new DameLoesung( loes4);
    DameLoesung loesung5 = new DameLoesung( loes5);
    DameLoesung loesung6 = new DameLoesung( loes6);

    System.out.println( loesung1);
    System.out.println( loesung2);
    System.out.println( loesung1.equals( loesung2));

    System.out.println( loesung1);
    System.out.println( loesung3);
    System.out.println( loesung1.equals( loesung3));

    System.out.println( loesung1);
    System.out.println( loesung4);
    System.out.println( loesung1.equals( loesung4));

    System.out.println( loesung1);
    System.out.println( loesung5);
    System.out.println( loesung1.equals( loesung5));

    System.out.println( loesung1);
    System.out.println( loesung6);
    System.out.println( loesung1.equals( loesung6));
  }
}